#version 120

#define FilmGrain
#define Saturation 1.5 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Contrast 1.4 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//#define chromaticaberration 
#define chromaticaberrationstrength 0.005 //[0.001 0.002 0.003 0.004 0.005 0.006 0.007 0.008 0.009 0.01]
#define tonemap 1 //[1 2]
uniform sampler2D tex;
uniform sampler2D depthtex1;
uniform sampler2D noisetex;

uniform float viewWidth;
uniform float viewHeight;
uniform float aspectRatio;
uniform float rainStrength;

varying vec2 texcoord;
varying vec4 color;

varying vec3 viewVector;
varying vec3 moonVec;

vec2 ca(vec2 coord) {
    float dist = distance(texcoord, vec2(0.5));
    float cas = chromaticaberrationstrength*dist;
    float r = texture2D(tex,texcoord.xy-vec2(cas,0)).r;
    float b = texture2D(tex,texcoord.xy+vec2(cas,0)).b;
    return vec2(r,b);
}

vec3 getSaturate(vec3 scolor){
    float mixrgb = (scolor.r + scolor.g + scolor.b) / 3.5;
    float weight = (Saturation-(0.5*rainStrength)) + (1.0 - pow(1.0 - 1.0 * mixrgb, 2.0)) * 0.08;

    return mix(vec3(mixrgb), scolor, weight);
}

vec3 getContrast(vec3 scolor, float consv) {
    consv = consv;
    vec3 l = log2(scolor+1e-4);
    l = (l +1.0)*consv-1.0;
    return exp2(l)-1e-4;
}

// Originally made by Richard Burgess-Dawson
// Modified by https://github.com/TechDevOnGitHub
float luminance(vec3 color) {
    return dot(color, vec3(0.212, 0.7152, 0.722));
}

vec3 burgess(vec3 color) {
    vec3 maxColor = color * min(vec3(1.0), 1.0 - exp(-1.0 / (luminance(color) * 0.1) * color));
    return (maxColor * (6.2 * maxColor + 0.5)) / (maxColor * (6.2 * maxColor + 1.7) + 0.06);
}

vec3 ACESFilm(vec3 x) //function by https://knarkowicz.wordpress.com/2016/01/06/aces-filmic-tone-mapping-curve/
{
    float a = 2.51;
    float b = 0.03;
    float c = 2.43;
    float d = 0.59;
    float e = 0.14;
    return clamp((x*(a*x+b))/(x*(c*x+d)+e),0.0,1.0);
}


void main() {
    float contrastvalue = Contrast;
    vec4 scolor = texture2D(tex, texcoord) * color;

    float depth = texture2D(depthtex1, texcoord).x;

    scolor.rgb = getSaturate(scolor.rgb);
    scolor.rgb = getContrast(scolor.rgb,contrastvalue);
    #ifdef chromaticaberration
    scolor.rb = ca(texcoord);
    #endif
    #if tonemap == 2
    scolor.rgb = burgess(scolor.rgb);
    #elif tonemap == 1
    scolor.rgb = ACESFilm(scolor.rgb);
    #endif
    scolor.rgb = pow(scolor.rgb, vec3(1.0 / 2.2));


/*DRAWBUFFERS:0*/
    gl_FragColor = scolor;

}

